;------------------------------------------------------------------------------
; UtilTool.sc
; Purpose: Presents a "Tools Menu" for PAL applications
;------------------------------------------------------------------------------
; Revision history:
; 08/26/91  Original version - without verify and rebuild options
;------------------------------------------------------------------------------

if not isassigned(lib_name) then
  lib_name = "paradox"
endif

if not isfile(lib_name+".lib") then
  createlib lib_name size 300
endif

? "UtilTool"

;-------------------------------------------------------------------------------
;                                                                   tools_menu()
;-------------------------------------------------------------------------------
; Purpose: Displays the "Tools" menu
proc tools_menu(
  help_proc_name,  ;The name of the help proc to be called if necessary
  clear_canvas,    ;Logical flag: if True, canvas will be cleared at start
  clear_workspace) ;Logical flag: if True, workspace will be cleared at start

  private
  old_errorproc,   ;The name of any pre-existing errorproc
  tools_choice,    ;The user's menu choice
  x,               ;Throwaway variable in which to store the user's keystroke
  proc_name        ;This proc's name, used for error logging

  if clear_workspace then clearall
  endif

  proc_name = "tools_menu"
  tools_choice = "Help"

  while true
    if clear_canvas then clear
    endif
    showmenu
      "Help"    : "Help on how to run this application",
      "Paradox" : "Exit out to native Paradox",
      "DOS"     : "Shell out to DOS",
      "Indexes" : "Maintain the tables' secondary indexes",
      "Rebuild" : "Rebuild one of the tables (use only if table is corrupted)",
      "Verify"  : "Verify a table to see if it is corrupted",
      "B&W"     : "Set monitor type setting to Black & White"
      default tools_choice
    to tools_choice
    working_message("")
    switch
      case tools_choice = "B&W":
        setbw paintcanvas attribute 52 0,0,23,0
      case tools_choice = "Verify" or
        tools_choice = "Rebuild":
        beep
        message "Menu choice not yet defined"
        sleep 1000
        loop
      case tools_choice = "Indexes":
        x=table_select(
        "v\\tables","1","table location",
        "Move the cursor to the table to work with "+
        "and then press F2 to select it.  ",
        "Press Esc to cancel the operation.")
        if x = "" then
          loop
        endif
        working_message("")
        index_mgr(x)
      case tools_choice = "Paradox":
        ;To prevent unexpected results if runing under Runtime:
        if isruntime() then
          user_message("Sorry, you cannot drop out to Paradox "+
          "when this application ",
          "is being run under Runtime.  Press any key to go on.")
          return false
        endif
        user_message("Press [Esc] to cancel, or any other key to go to "+
        "native Paradox.","Press any key to go on.",0)
        if retval = 27 then
          loop
        else
          reset
          quit
        endif
      case tools_choice = "DOS":
        user_message("Press [Esc] to cancel, or any Tools key to go to "+
        "DOS.  To return to ",
        "the application from DOS, type Exit and press Enter.",0)
        if retval = 27 then
          loop
        else
          reset
          cursor normal
          dosbig
        endif
      case tools_choice = "Help" :
        if isassigned(errorproc) then
          old_errorproc = errorproc
        endif
        errorproc = "xerrorproc"
        proc xerrorproc()
          beep
          user_message("Sorry, there is no help available.",
          "Press any key to go on.",0)
          return 1
        endproc
        execproc help_proc_name
        if isassigned(old_errorproc) then
          errorproc = old_errorproc
        endif
        release procs xerrorproc
      case tools_choice = "Esc"  :
        return
      otherwise : beep
        loop
    endswitch
  endwhile

endproc

writelib lib_name tools_menu
release procs tools_menu
?? "."

;-------------------------------------------------------------------------------
;                                                                 table_select()
;-------------------------------------------------------------------------------
; purpose: Used to present the user with a list of tables for selection, and,
; if user selects one, returns the location of that table.  If not, returns "".
proc table_select(
  list_table_name,  ;The name of the table to be displayed
  list_table_form,  ;The name of the form to be used
  list_table_field, ;The name of the field whose value should be returned
  prompt_1,         ;The text to display on the first prompt line
  prompt_2)         ;The text to display on the second prompt line

  private
  userkey,          ;The user's keypress
  proc_name,        ;The name of this proc, for error logging purposes
  table_choice      ;The user's table selection

  proc_name = "table_select"

  clear
  working_message("")
  view list_table_name
  pickform list_table_form
  while true
    wait table
      prompt prompt_1,prompt_2
    until "Dos","DosBig","FieldView","F2","Esc"
    userkey = retval
    if userkey = "F2" or userkey = "Esc" then
      clear
      working_message("")
      execute "table_choice = ["+list_table_field+"]"
      if userkey = "Esc" then
        clearimage
        table_choice = ""
      endif
      return table_choice
    else
      beep
      loop
    endif
  endwhile

endproc

writelib lib_name table_select
release procs table_select
?? "."

;-------------------------------------------------------------------------------
;                                                                   index_mgr()
;-------------------------------------------------------------------------------
; purpose: To manage secondary indexes on tables.  Throughout this module,
; "indexes" refers to *secondary* indexes
proc index_mgr(table_to_use) ; The name of the table we're doing this for

  private
  proc_name,        ; This proc's name (for errorproc purposes)
  indexes,          ; Array holding names of fields with indexes
  invalid,          ; Array holding flags for non-current indexes
  field_name,       ; Name of a field with an index
  worktable,        ; Name of a temporary table in the privdir
  is_keyed,         ; Logical flag: true if table_to_use is keyed
  n,                ; Counter for various purposes
  indexes_exist,    ; Logical flag: there are pre-existing indexes
  index_to_delete,  ; Name of index files to be deleted
  count_n           ; Counter, size of indexes and of new indexes to build

  proc_name = "index_mgr"
  working_message("Checking table's structure.")
  worktable = privdir()+"indxtemp"              ; Initialize the variable
  if istable(worktable) then                    ; Get rid of old copy, if any
    delete worktable
  endif

  if nkeyfields(table_to_use) <> 0 then         ; Initialize is_keyed
    is_keyed = true
  else
    is_keyed = false
  endif

  get_indexes()                                 ; Get the current index status
  if not retval then
    return false
  endif

  moveto "v\\tables" clearimage                 ; Clean up after ourselves
  moveto worktable                              ; Go back to the worktable

  mark_indexes()                                ; Tell us what index work to do
  if not retval then
    return false
  endif

  reset

  view worktable
  working_message("")
  home moveto [Set Index]                     ; Get to starting position
  n = 0                                       ; Re-initialize the variable
  scan for not isblank([])
    n = n + 1                                ; Increment the counter
    @ 01,00                                  ; Entertain the user
    ?? "Working on task "+strval(n)+" of "+strval(count_n)+"."
    ;-- Can't make maintained indexes unless table is keyed
    if [] = "" then                         ; User wanted an index created
      if is_keyed then
        index maintained table_to_use on [field name]
      else
        index table_to_use on [field name]
      endif
    else                                     ; User wanted an index deleted
      index_to_delete = table_to_use+".?" + decimal_to_hex(recno())
      run norefresh "del "+index_to_delete  ; Nuke the .x and .y files
    endif
  endscan
  delete worktable                            ; Get rid of the temporary table
  return true
endproc

writelib lib_name index_mgr
release procs index_mgr
?? "."

;-------------------------------------------------------------------------------
;                                                                 get_indexes()
;-------------------------------------------------------------------------------
; purpose: A sub-module for index_mgr().   Detetmines if there are any
; pre-existing secondary indices and sets up the private, temporary table used
; for the rest of the index_mgr() operations.
proc get_indexes()

  private
  proc_name

  proc_name = "get_indexes"

  working_message("Checking to see if there are already any existing indexes.")

  ;Get the family for table_to_use and see if there are already any indexes
  menu {tools} {info} {family}    ;Get family of the table
  select table_to_use
  menu {ask}                      ;Get rid of any non-index family members
  select "family"
  checkplus
  [Name] = "Speedup for.."
  do_it!

  if isempty("Answer") then  ; There are no existing indexes!
    indexes_exist = false
  else                       ; There ARE existing indexes!
    indexes_exist = true
  endif

  rename "answer" "family"
  moveto "family"                              ;Get back to the right table

  if indexes_exist then
    moveto [name]                             ;Get back to the right field
    array indexes[ nrecords(table()) ]        ;Initialize the index
    array invalid[ nrecords(table()) ]        ;Initialize the index
    scan
      field_name = ""                        ;Initialize the variable
      ;See if not current or corrupted
      if match( [], "..[Not current]..", field_name ) then
        invalid[recno()] = "Not valid: Rebuild or Delete"
      endif

      ;Trim off the "[Maintained]"
      if field_name = "" then field_name = []
      endif
      while match( field_name, "..[Maintained]..", field_name )
      endwhile

      ;Trim off trailing spaces
      while match( field_name, ".. ", field_name )
      endwhile
      indexes[recno()] = field_name

      ;Trim off leading "Speedup for"
      while match( field_name, "Speedup for ..", field_name )
      endwhile
      indexes[recno()] = field_name
    endscan
  endif

  clearimage                               ; Get rid of the "family" table

  working_message("Preparing a table for you to use to "+
  "issue your index management instructions.")

  ;Get the structure of table_to_use and
  ;modify "struct" to look like we want it to
  menu {Tools} {Info} {Structure}          ;Get the structure
  select table_to_use

  menu {Tools} {Rename} {Table} {Struct}   ;Rename "struct"
  select worktable
  if menuchoice() = "Cancel" then {Replace}
  endif
  menu {Modify} {Restructure}              ;Modify worktable
  select worktable
  down del down                            ;Get rid of [field type]
  "Set index" enter  "a1" enter            ;Add field
  "Index already exists" enter  "a35"      ;Add field
  do_it!                                   ;Complete the restructure

  if menuchoice() = "Delete" then {Delete}
  endif
  ;-- worktable is left on the workspace after this is finished

  if not indexes_exist then  ;We can quit here if there weren't any
    return true              ;pre-existing indexes!
  endif

  working_message("Filling in the information about any existing indexes.")

  ;-- Fill in the info about any existing indexes
  editkey                                      ;Go into Edit mode
  moveto [Set index]
  menu {Valcheck} {Define} enter {Picture} {} ;Set up the  char requirement
  count_n = arraysize(indexes)
  moveto [Field name]
  scan for true                    ;Fill in the "Index already exists" column
    for n from 1 to count_n
      if isassigned(indexes[n]) and lower([]) = lower(indexes[n]) then
        if isassigned(invalid[n]) then
          [Index already exists] = "Yes"+" "+invalid[n]
        else
          [Index already exists] = "Yes"
        endif
        loop
      endif
    endfor
  endscan
  do_it!
  return true
endproc

writelib lib_name get_indexes
release procs get_indexes
?? "."

;-------------------------------------------------------------------------------
;                                                                 mark_indexes()
;-------------------------------------------------------------------------------
; purpose: handles the WAIT for the user to mark the worktable during
; index_mgr() to indicate what he or she wants done
proc mark_indexes()

  private
  proc_name,      ; This proc's name (for errorproc purposes)
  x               ; Throwaway menu choice

  proc_name = "mark_indexes"

  firstshow                       ; Let's just show them worktable
  editkey                         ; Switch to Edit mode
  home                            ; Move to the top of the table
  moveto [#]                      ; Move to the record # column
  menu {image} {columnsize} enter ; Resize the record # column to
  home right right enter          ;   minimize its width
  moveto [Set index]              ; Move to the  field
  while true
    wait field
      prompt "[F6]:toggle \"\" mark   [D]:toggle \"D\" mark    [F2]:finished   [Esc]:cancel",
      " mark: create an index for the field    D mark: delete the index for this field"
    until "Esc", "F2", "Enter", "Up", "Down", "PgUp", "PgDn",
    68,  ; "D"
    100, ; "d"
    "Home", "End", "Rotate", "F35", "Fieldview", "F6",
    "Dos", "Dosbig"
    switch
      case retval = 68 or retval = 100:  ; User pressed "D" or "d"
        if isblank([]) then
          [] = "D"
        else
          ctrlbackspace
        endif
      case retval = "F6":
        if isblank([]) then
          [] = ""
        else
          ctrlbackspace
        endif
      case retval = "Esc"  :
        working_message("Cancelling operation.")
        canceledit
        delete worktable
        clearall
        return false
      case retval = "F2"   :
        showmenu
          "No"  : "No, you are NOT ready to go on",
          "Yes" : "Yes, you ARE ready to go on"
        to x
        if x = "No" or x = "Esc" then
          loop
        endif
        working_message("")
        count_n = 0                       ; Initialize the variable
        scan for not isblank([set index]) ; Count up the indexes to build
          count_n = count_n + 1
        endscan
        do_it!
        quitloop
      case retval = "Up"   :
        if recno() = 1 then
          beep
        else
          up
        endif
      case retval = "Down" :
        if recno() <> nimagerecords() then
          down
        else
          beep
        endif
      case retval = "PgUp" : pgup
      case retval = "PgDn" : pgdn
      case retval = "Home" : home
      case retval = "End"  : end
      otherwise          : beep
    endswitch
  endwhile
  return true
endproc

writelib lib_name mark_indexes
release procs mark_indexes
?? "."

;-------------------------------------------------------------------------------
;                                                                 user_message()
;-------------------------------------------------------------------------------
; purpose: displays a message on lines one and two and, optionally, beeps
; the speaker at the user in a controllable fashion
proc user_message(
  prompt_1, ;The text to display on the first prompt line
  prompt_2, ;The text to display on the second prompt line
  beeper)   ;0=no beep, 1=one beep, otherwise, milliseconds between beeps

  private userkey,      ;User's key presses
  proc_name     ;This proc's name, used for error logging purposes

  proc_name = "user_message"

  @ 0,0 clear eol ?? format("w80,al",prompt_1) ;Present the messages
  @ 1,0 clear eol ?? format("w80,al",prompt_2) ;to the user
  cursor off                                   ;Turn off the blinking cursor

  while charwaiting()   ;Throw away any characters that the
    userkey=getchar()  ;user may have typed ahead in order to
  endwhile              ;make sure that these messages get read

  switch
    case beeper = 0:
      return getchar()
    case beeper = 1: beep
      return getchar()
    otherwise:
      while not charwaiting()
        beep
        sleep beeper
      endwhile
      return getchar()
  endswitch
endproc

writelib lib_name user_message
release procs user_message
?? "."

;-------------------------------------------------------------------------------
;                                                            working_message("")
;-------------------------------------------------------------------------------
; Purpose: Displays a generic "Working" message followed by blinking periods
;          at 0,0 and optionally displays a user-defined message at
;          1,0
proc working_message(message_text)
  private proc_name  ;This proc's name, used for error logging purposes
  proc_name = "working_message"

  style
  @ 00,00 clear eol ?? "Working"
  style blink ?? "..."  style
  @ 01,00 clear eol ?? message_text
  cursor off

endproc

writelib lib_name working_message
release procs working_message
?? "."

;------------------------------------------------------------------------------
;                                                              decimal_to_hex()
;------------------------------------------------------------------------------
; Purpose: converts a decimal value between 0 and 255 to it's 2 digit hex value.
; Based upon a version originally uploaded to the BorDb forum on Compuserve
; by Alan Zenreich
; Decimal2Hex.a(123) returns "7B"

proc decimal_to_hex(n) ; n is decimal value<=255
  private a
  a = "0123456789ABCDEF"
  return substr(a, 1+int(n/16),1) + substr(a, 1+mod(n,16),1)
endproc
writelib lib_name decimal_to_hex
release procs decimal_to_hex
?? "."


